#!/usr/bin/env python
# coding: utf-8
#
# Copyright 2019 by Leipzig University Library, http://ub.uni-leipzig.de
#                   The Finc Authors, http://finc.info
#                   Martin Czygan, <martin.czygan@uni-leipzig.de>
#                   Robert Schenk, <robert.schenk@uni-leipzig.de>
#
# This file is part of some open source application.
#
# Some open source application is free software: you can redistribute
# it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, either
# version 3 of the License, or (at your option) any later version.
#
# Some open source application is distributed in the hope that it will
# be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
#
# @license GPL-3.0+ <http://spdx.org/licenses/GPL-3.0+>
"""

Source: Universitätsbibliothek Frankfurt am Main (VK Film)
SID: 119
Ticket: #8571, #14654, #15877

"""

import io
import re
import sys

import marcx
import pymarc
from siskin.utils import marc_clean_record


def check_fulltext(record):
    """
    Checks several fields and returns True, if there is probably a link to fulltext.
    """
    try:
        type = record["024"]["2"]
    except:
        type = ""

    try:
        urls = record.get_fields("856")
    except:
        urls = ""

    for url in urls:
        description = url.get_subfields("3")
        if not description or "kostenfrei" in description or "Digitalisierung" in description or "zugänglich" in description:
            has_fulltext_url = True
            break
    else:
        has_fulltext_url = False

    if (urls and has_fulltext_url) or (urls and (format == "doi" or format == "urn")):
        return True
    else:
        return False


inputfilename = "119_input.mrc"
outputfilename = "119_output.mrc"

if len(sys.argv) == 3:
    inputfilename, outputfilename = sys.argv[1:]

inputfile = open(inputfilename, "rb")
outputfile = open(outputfilename, "wb")

reader = pymarc.MARCReader(inputfile)

for record in reader:

    record = marcx.Record.from_record(record)
    record.force_utf8 = True
    record.strict = False

    # Identifikator
    try:
        f001 = record["001"].data
    except AttributeError:
        continue
    record.remove_fields("001")
    record.add("001", data="finc-119-" + f001)

    # Zugangsfacette
    has_fulltext = check_fulltext(record)
    try:
        f007 = record["007"].data
    except:
        f007 = ""

    if f007 and not has_fulltext:
        if f007.startswith("cr"):
            record.remove_fields("007")
            record.add("007", data="tu")
    elif has_fulltext:
        record.remove_fields("007")
        record.add("007", data="cr")
    elif not f007:
        record.add("007", data="tu")

    # Kollektion
    record.remove_fields("912")
    record.add("912", a="vkfilm")

    # Ansigelung
    collections = ["a", f001, "b", "119", "c", "sid-119-col-ubfrankfurt"]
    record.add("980", subfields=collections)

    marc_clean_record(record)
    outputfile.write(record.as_marc())

outputfile.close()



#!/usr/bin/env python
# coding: utf-8
#
# Copyright 2019 by Leipzig University Library, http://ub.uni-leipzig.de
#                   The Finc Authors, http://finc.info
#                   Martin Czygan, <martin.czygan@uni-leipzig.de>
#                   Robert Schenk, <robert.schenk@uni-leipzig.de>
#
# This file is part of some open source application.
#
# Some open source application is free software: you can redistribute
# it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, either
# version 3 of the License, or (at your option) any later version.
#
# Some open source application is distributed in the hope that it will
# be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
#
# @license GPL-3.0+ <http://spdx.org/licenses/GPL-3.0+>

"""

Source: RISM
SID: 14
Ticket: 1145, #4435, #16018

"""

import io
import sys
from builtins import *

import marcx
import pymarc
from siskin.mappings import formats
from siskin.utils import marc_clean_record


copytags = ("003", "004", "005", "006", "008", "009", "010", "011", "012", "013", "014", "015", "016", "017", "018",
            "019", "020", "021", "022", "023", "024", "025", "026", "027", "028", "029", "030", "031", "032", "033",
            "034", "035", "036", "037", "038", "039", "040", "041", "042", "043", "044", "045", "046", "047", "048",
            "049", "050", "051", "052", "053", "054", "055", "056", "057", "058", "059", "060", "061", "062", "063",
            "064", "065", "066", "067", "068", "069", "070", "071", "072", "073", "074", "075", "076", "077", "078",
            "079", "080", "081", "082", "083", "085", "086", "087", "088", "089", "090", "091", "092", "093", "094",
            "095", "096", "097", "098", "099", "100", "101", "102", "103", "104", "105", "106", "107", "108", "110",
            "111", "112", "113", "114", "115", "116", "117", "118", "120", "121", "124", "125", "126", "129", "130",
            "131", "134", "135", "138", "142", "145", "148", "149", "151", "155", "156", "157", "160", "162", "163",
            "164", "165", "166", "167", "173", "174", "175", "176", "178", "183", "184", "187", "192", "200", "204",
            "210", "212", "234", "240", "243", "254", "256", "260", "300", "340", "383", "386", "387", "447", "448",
            "456", "490", "500", "504", "510", "518", "520", "541", "561", "563", "590", "591", "592", "593", "594",
            "595", "596", "597", "650", "657", "700", "710", "730", "762", "773", "775", "787")


def has_digitalization_links(record):
    for field_856 in record.get_fields("856"):
        for x in field_856.get_subfields("x"):
            if "Digitalization" == x:
                return True
        for z in field_856.get_subfields("z"):
            if "digit" in z or "Digit" in z and not ("Bach digital" in z or "Bach Digital" in z):
                return True
            if "exemplar" in z or "Exemplar" in z:
                return True

def get_digitalization_links(record):
    """
    Given a marc record, just return all links (from 856.u).

    Same as:

        >>> record = marcx.Record()
        >>> record.add('856', u='http://google.com')
        >>> list(record.itervalues('856.u'))
        ['http://google.com']

    """
    for field_856 in record.get_fields("856"):
        is_digitalization = False
        for x in field_856.get_subfields("x"):
            if "Digitalization" == x:
                is_digitalization = True
        if not is_digitalization:
            for z in field_856.get_subfields("z"):
                if "digit" in z or "Digit" in z and not ("Bach digital" in z or "Bach Digital" in z):
                    is_digitalization = True
                elif "exemplar" in z or "Exemplar" in z:
                    is_digitalization = True
        if is_digitalization:
            for url in field_856.get_subfields("u"):
                yield url


def get_titles(record):
    for field in record.get_fields("772"):
        for title in field.get_subfields("a"):
            yield title


def get_field(record, field, subfield="a"):
    """
    Shortcut to get a record field and subfield or empty string, if no value is found.
    """
    try:
        value = record[field][subfield]
        return value if value is not None else ""
    except (KeyError, TypeError):
        return ""


# Default input and output.
inputfilename, outputfilename = "14_input.mrc", "14_output.mrc"

if len(sys.argv) == 3:
    inputfilename, outputfilename = sys.argv[1:]

inputfile = io.open(inputfilename, "rb")
outputfile = io.open(outputfilename, "wb")
reader = pymarc.MARCReader(inputfile)

format = "Score"

for oldrecord in reader:

    newrecord = marcx.Record()
    newrecord.force_utf8 = True
    newrecord.strict = False

    # prüfen, ob Titel vorhanden ist
    f245 = oldrecord["245"]
    if not f245:
        continue

    # prüfen, ob es sich um Digitalisat handelt bzw. ein Link zu einem Digitalisat enthalten ist
    f856 = oldrecord["856"]
    if not f856 or not has_digitalization_links(oldrecord):
        continue

    for field in oldrecord.get_fields("856"):
        f856 = field if "http" in field else ""

    # leader
    leader = formats[format]["Leader"]
    newrecord.leader = leader

    # 001
    f001 = oldrecord["001"].data
    newrecord.add("001", data="finc-14-%s" % f001)

    # Zugangsfacette
    f007 = formats[format]["e007"]
    newrecord.add("007", data=f007)

    # Originalfelder, die ohne Änderung übernommen werden
    for tag in copytags:
        for field in oldrecord.get_fields(tag):
            newrecord.add_field(field)

    # Haupttitel
    f240a = get_field(oldrecord, "240", "a")
    f240m = get_field(oldrecord, "240", "m")
    f240n = get_field(oldrecord, "240", "n")
    f245a = "%s %s %s" % (f240a, f240m, f240n)
    newrecord.add("245", a=f245a)

    # Alternativtitel
    f246a = get_field(oldrecord, "245", "a")
    if f246a != "[without title]":
        newrecord.add("246", a=f246a)

    # RDA-Inhaltstyp
    f336b = formats[format]["336b"]
    newrecord.add("336", b=f336b)

    # RDA-Datenträgertyp
    f338b = formats[format]["338b"]
    newrecord.add("338", b=f338b)

    # Fußnote
    f852a = get_field(oldrecord, "852", "a")
    f852c = get_field(oldrecord, "852", "c")  # häufig None
    f852x = get_field(oldrecord, "852", "x")
    f500a = "%s, %s, %s" % (f852a, f852c, f852x)
    newrecord.add("500", a=f500a)

    # enthaltene Werke
    try:
        titles = list(get_titles(oldrecord))
        if titles:
            for title in titles:
                newrecord.add("505", a=title)
    except (KeyError, TypeError):
        pass

    # 856 (Digitalisat)
    digitalization_links = list(get_digitalization_links(oldrecord))
    if len(digitalization_links) > 0:
        for digitalization_link in digitalization_links:
            newrecord.add("856", q="text/html", _3="Link zum Digitalisat", u=digitalization_link)

    # 856 (Datensatz)
    newrecord.add("856", q="text/html", _3="Link zum Datensatz", u="https://opac.rism.info/search?id=" + f001)

    # SWB-Inhaltstyp
    f935c = formats[format]["935c"]
    newrecord.add("935", c=f935c)

    # 970
    newrecord.add("970", c="PN")

    # Ansigelung
    newrecord.add("980", a=f001, b="14", c="sid-14-col-rism")

    marc_clean_record(newrecord)
    outputfile.write(newrecord.as_marc())

inputfile.close()
outputfile.close()



# coding: utf-8
# pylint: disable=C0301,E1101

# Copyright 2019 by Leipzig University Library, http://ub.uni-leipzig.de
#                   The Finc Authors, http://finc.info
#                   Robert Schenk, <robert.schenk@uni-leipzig.de>
#                   Martin Czygan, <martin.czygan@uni-leipzig.de>
#
# This file is part of some open source application.
#
# Some open source application is free software: you can redistribute
# it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, either
# version 3 of the License, or (at your option) any later version.
#
# Some open source application is distributed in the hope that it will
# be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
#
# @license GPL-3.0+ <http://spdx.org/licenses/GPL-3.0+>
"""
NL.

Source: 17
Ticket: 15865

Config

[nl]

solr = https://example.com/vaqrk/FYHO
"""

import datetime

import luigi
from gluish.common import Executable
from gluish.intervals import monthly
from gluish.parameter import ClosestDateParameter
from gluish.utils import shellout
from siskin.task import DefaultTask


class NLTask(DefaultTask):
    """
    Base task.
    """
    TAG = '17'

    def closest(self):
        return monthly(date=self.date)


class NLFetch(NLTask):
    """
    Stream from SOLR.
    """
    date = ClosestDateParameter(default=datetime.date.today())
    query = luigi.Parameter(default="collection_details:GBV_NL_EBOOK",
                            description="to test: id:NLEB006936695 OR id:NLEB006936733")

    def run(self):
        """
        cf. https://github.com/stedolan/jq/issues/787, "Warning: replace is deprecated and will be removed in a future version."

        There's jq -j, but replace cannot run on a single 1GB line.
        """
        output = shellout("""solrdump -verbose -server {server} -q "{query}" -fl fullrecord | \
                          jq -r '.fullrecord' | \
                          replace '#29;' $(printf "\\x1D") '#30;' $(printf "\\x1E") '#31;' $(printf "\\x1F") > {output} """,
                          server=self.config.get('nl', 'solr'),
                          query=self.query)
        output = shellout("sed ':a;N;$!ba;s/\\x1d\\x0a/\\x1d/g' {input} > {output}", input=output)
        luigi.LocalTarget(output).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext="mrc", digest=True))




# coding: utf-8
# pylint: disable=C0301,E1101

# Copyright 2019 by Leipzig University Library, http://ub.uni-leipzig.de
#                   The Finc Authors, http://finc.info
#                   Martin Czygan, <martin.czygan@uni-leipzig.de>
#                   Robert Schenk, <robert.schenk@uni-leipzig.de>
#
# This file is part of some open source application.
#
# Some open source application is free software: you can redistribute
# it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, either
# version 3 of the License, or (at your option) any later version.
#
# Some open source application is distributed in the hope that it will
# be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
#
# @license GPL-3.0+ <http://spdx.org/licenses/GPL-3.0+>
"""
KXP, FID-BBI

Tickets: 15833, 15694
"""

import datetime

import luigi
from gluish.common import Executable
from gluish.intervals import weekly
from gluish.parameter import ClosestDateParameter
from gluish.utils import shellout
from siskin.task import DefaultTask


class KXPTask(DefaultTask):
    """
    Base task for KXP. TAG might change from kxp to numeric id in the future.
    """
    TAG = 'kxp'

    def closest(self):
        return weekly(date=self.date)


class KXPSRU(KXPTask):
    """
    Fetch selection via SRU.
    """
    date = ClosestDateParameter(default=datetime.date.today())

    def requires(self):
        return Executable(name='srufetch', message='https://github.com/miku/srufetch')

    def run(self):
        # XXX: Move this to config, then gitlab.
        selector = """
            pica.bkl="05.15" or pica.bkl="05.30" or pica.bkl="05.33" or
            pica.bkl="05.38" or pica.bkl="05.39" or pica.bkl="06.*" or
            pica.bkl="17.80" or pica.bkl="17.91" or pica.bkl="17.94" or
            pica.bkl="21.30" or pica.bkl="21.31" or pica.bkl="21.32" or
            pica.bkl="21.93" or pica.bkl="53.70" or pica.bkl="53.71" or
            pica.bkl="53.72" or pica.bkl="53.73" or pica.bkl="53.74" or
            pica.bkl="53.75" or pica.bkl="53.76" or pica.bkl="53.82" or
            pica.bkl="53.89" or pica.bkl="54.08" or pica.bkl="54.10" or
            pica.bkl="54.22" or pica.bkl="54.32" or pica.bkl="54.38" or
            pica.bkl="54.51" or pica.bkl="54.52" or pica.bkl="54.53" or
            pica.bkl="54.54" or pica.bkl="54.55" or pica.bkl="54.62" or
            pica.bkl="54.64" or pica.bkl="54.65" or pica.bkl="54.72" or
            pica.bkl="54.73" or pica.bkl="54.74" or pica.bkl="54.75" or
            pica.bkl="86.28" or pica.ssg="24,1" or pica.ssg="bbi" or
            pica.sfk="bub" or pica.osg="bbi" or pica.rvk="AM 1*" or
            pica.rvk="AM 2*" or pica.rvk="AM 3*" or pica.rvk="AM 4*" or
            pica.rvk="AM 5*" or pica.rvk="AM 6*" or pica.rvk="AM 7*" or
            pica.rvk="AM 8*" or pica.rvk="AM 9*" or pica.rvk="AN 1*" or
            pica.rvk="AN 2*" or pica.rvk="AN 3*" or pica.rvk="AN 4*" or
            pica.rvk="AN 5*" or pica.rvk="AN 6*" or pica.rvk="AN 7*" or
            pica.rvk="AN 8*" or pica.rvk="AN 9*" or pica.rvk="AP 25*" or
            pica.rvk="AP 28*" or pica.rvk="LH 72*" or pica.rvk="LK 86*" or
            pica.rvk="LR 5570*" or pica.rvk="LR 5580*" or pica.rvk="LR 5581*"
            or pica.rvk="LR 5582*" or pica.rvk="LR 5586*" or pica.rvk="LR
            57240" or pica.rvk="LT 5570*" or pica.rvk="LT 5580*" or
            pica.rvk="LT 5581*" or pica.rvk="LT 5582*" or pica.rvk="LT 5586*"
            or pica.rvk="LT 57240" or pica.sbn="vd17" or pica.sbn="vd18"
        """
        output = shellout("""srufetch -x -q '{selector}' > {output} """, selector=selector)
        output = shellout("""yaz-marcdump -i marcxml -o marc {input} > {output}""", input=output)
        luigi.LocalTarget(output).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext="mrc"))



#!/usr/bin/env python
# coding: utf-8
#
# Copyright 2019 by Leipzig University Library, http://ub.uni-leipzig.de
#                   The Finc Authors, http://finc.info
#                   Robert Schenk, <robert.schenk@uni-leipzig.de>
#
# This file is part of some open source application.
#
# Some open source application is free software: you can redistribute
# it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, either
# version 3 of the License, or (at your option) any later version.
#
# Some open source application is distributed in the hope that it will
# be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
#
# @license GPL-3.0+ <http://spdx.org/licenses/GPL-3.0+>

"""

Source: Umweltbibliothek Leipzig
SID: 156
Ticket: #11786

"""

import re
import sys

import marcx
import pymarc
from siskin.mappings import formats
from siskin.utils import marc_clean_record, check_isbn, check_issn

copytags = ("100", "105", "120", "130", "150", "174", "200", "245", "246", "250", "260", "300",
            "335", "351", "361", "400", "500", "520", "650", "689", "700", "710", "800")

inputfilename = "156_input.xml"
outputfilename = "156_output.mrc"

if len(sys.argv) == 3:
    inputfilename, outputfilename = sys.argv[1:]

inputfile = open(inputfilename, "rb")
outputfile = open(outputfilename, "wb")
oldrecords = pymarc.parse_xml_to_array(inputfile)

for i, oldrecord in enumerate(oldrecords, start=1):

    try:
        f245a = oldrecord["245"]["a"]
    except:
        continue

    newrecord = marcx.Record(force_utf8=True)
    newrecord.strict = False

    # pauschale Festlegung
    format = "Book"

    # leader
    leader = formats[format]["Leader"]
    newrecord.leader = leader

    # Identifikator
    f001 = str(i)
    newrecord.add("001", data="finc-156-" + f001)

    # Zugangsart
    f007 = formats[format]["p007"]
    newrecord.add("007", data=f007)

    # ISBN
    try:
        isbn = oldrecord["020"]["a"]
    except:
        isbn = ""
    f020a = check_isbn(isbn)
    newrecord.add("020", a=f020a)

    # ISSN
    try:
        issn = oldrecord["022"]["a"]
    except:
        issn = ""
    f022a = check_issn(issn)
    newrecord.add("022", a=f022a)

    # RDA-Inhaltstyp
    f336b = formats[format]["336b"]
    newrecord.add("336", b=f336b)

    # RDA-Datenträgertyp
    f338b = formats[format]["338b"]
    newrecord.add("338", b=f338b)

    # Originalfelder, die ohne Änderung übernommen werden
    for tag in copytags:
        for field in oldrecord.get_fields(tag):
            newrecord.add_field(field)

    # 980
    collections = ["a", f001, "b", "156", "c", "sid-156-col-umweltbibliothek"]
    newrecord.add("980", subfields=collections)

    marc_clean_record(newrecord)
    outputfile.write(newrecord.as_marc())

inputfile.close()
outputfile.close()



# coding: utf-8
# pylint: disable=C0301,E1101

# Copyright 2019 by Leipzig University Library, http://ub.uni-leipzig.de
#                   The Finc Authors, http://finc.info
#                   Martin Czygan, <martin.czygan@uni-leipzig.de>
#
# This file is part of some open source application.
#
# Some open source application is free software: you can redistribute
# it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, either
# version 3 of the License, or (at your option) any later version.
#
# Some open source application is distributed in the hope that it will
# be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
#
# @license GPL-3.0+ <http://spdx.org/licenses/GPL-3.0+>
"""
LISSA, https://lissarchive.org/

> A free, open scholarly platform for library and information science,
https://osf.io/preprints/lissa/discover

Source: 179
Ticket: 15839
"""

import datetime
import itertools
import json

import langdetect
from iso639 import languages

import luigi
from gluish.format import Gzip
from gluish.intervals import weekly
from gluish.parameter import ClosestDateParameter
from gluish.utils import shellout
from siskin.task import DefaultTask


class LissaTask(DefaultTask):
    """
    Base task.
    """
    TAG = '179'

    def closest(self):
        return weekly(date=self.date)


class LissaFetch(LissaTask):
    """
    Fetch dataset via single curl request with large size to ES.

    Fixed endpoint at: https://share.osf.io/api/v2/search/creativeworks/

    In 08/2019, there are only about 230 items, so a single request window of
    1000 suffices. Adjust, if dataset grows.
    """
    date = ClosestDateParameter(default=datetime.date.today())

    def run(self):
        es = "https://share.osf.io/api/v2/search/creativeworks"
        query = "sources:%22LIS%20Scholarship%20Archive%22"
        output = shellout("""curl -s --fail "{es}/_search?from=0&size=1000&q={query}" > {output}""", query=query, es=es)
        luigi.LocalTarget(output).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext="json"))


class LissaIntermediateSchema(LissaTask):
    """
    Convert custom JSON to intermediate schema.
    """
    date = ClosestDateParameter(default=datetime.date.today())

    def requires(self):
        return LissaFetch(date=self.date)

    def run(self):
        with self.input().open() as handle:
            resp = json.load(handle)

        converted = []

        for hit in resp["hits"]["hits"]:
            source = hit["_source"]
            doc = {
                "finc.id": "ai-179-{}".format(source["id"]),
                "finc.source_id": "179",
                "finc.format": "ElectronicArticle",
                "finc.record_id": source["id"],
                "finc.mega_collection": ["LISSA", "sid-179-col-lissa"],
                "ris.type": "EJOUR",
                "rft.atitle": source["title"],
                "rft.genre": "article",
                "rft.pub": source.get("publishers", []),
                "authors": [{
                    "rft.au": name
                } for name in source["contributors"]],
                "url": [link for link in source["identifiers"] if link.startswith("http")],
                "abstract": source.get("description", ""),
            }

            dois = [v.replace("http://dx.doi.org/", "") for v in source["identifiers"] if "doi.org" in v]
            if len(dois) == 0:
                self.logger.warn("document without DOI")
            elif len(dois) == 1:
                doc.update({"doi": dois[0]})
            else:
                # In 08/2019, various DOI seem to work.
                self.logger.warn("document with multiple dois: %s", dois)
                doc.update({"doi": dois[0]})

            if doc.get("language"):
                doc.update({"language": doc.get("language")})
            else:
                if len(doc["abstract"]) > 20:
                    result = langdetect.detect(doc["abstract"])
                    doc["languages"] = [languages.get(alpha2=result).bibliographic]
                    self.logger.debug("detected %s in abstract (%s)", doc["languages"], doc["abstract"][:40])

            # Gather subjects.
            subjects = source.get("subjects", []) + source.get("subject_synonyms", []) + source.get("tags", [])
            unique_subjects = set(itertools.chain(*[v.split("|") for v in subjects]))
            doc.update({"x.subjects": list(unique_subjects)})

            # Try date_published, then date_created, then fail.
            for key in ("date_published", "date_created"):
                if key not in source or not source[key]:
                    continue
                doc.update({
                    "x.date": source[key][:19] + "Z",
                    "rft.date": source[key][:10],
                })
                break
            else:
                raise ValueError("did not find any date field in document", hit)

            converted.append(doc)

        with self.output().open('w') as output:
            for doc in converted:
                serialized = json.dumps(doc) + "\n"
                output.write(serialized.encode("utf-8"))

    def output(self):
        return luigi.LocalTarget(path=self.path(ext="ldj.gz"), format=Gzip)



# coding: utf-8
# pylint: disable=C0301,E1101

# Copyright 2016 by Leipzig University Library, http://ub.uni-leipzig.de
#                   The Finc Authors, http://finc.info
#                   Martin Czygan, <martin.czygan@uni-leipzig.de>
#
# This file is part of some open source application.
#
# Some open source application is free software: you can redistribute
# it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, either
# version 3 of the License, or (at your option) any later version.
#
# Some open source application is distributed in the hope that it will
# be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
#
# @license GPL-3.0+ <http://spdx.org/licenses/GPL-3.0+>
"""
BASE, refs #5994.

* https://www.base-search.net/
* https://api.base-search.net/
* https://en.wikipedia.org/wiki/BASE_(search_engine)
* http://www.dlib.org/dlib/june04/lossau/06lossau.html

Currently (Dec 2018) BASE is provided via SLUB download.

Config
------

[base]

ftp-host = sftp://example.com
ftp-base = /target_data/xyz
ftp-username= admin
ftp-password= s3cr3t
ftp-pattern = *
"""

import datetime
import os

import luigi
from gluish.format import TSV, Gzip
from gluish.utils import shellout
from siskin.common import FTPMirror
from siskin.task import DefaultTask


class BaseTask(DefaultTask):
    """
    Various tasks around base, refs #14947.
    """
    TAG = '126'


class BasePaths(BaseTask):
    """
    Mirror SLUB FTP for base.
    """
    date = luigi.DateParameter(default=datetime.date.today())
    max_retries = luigi.IntParameter(default=10, significant=False)
    timeout = luigi.IntParameter(default=20, significant=False,
                                 description='timeout in seconds')

    def requires(self):
        return FTPMirror(host=self.config.get('base', 'ftp-host'),
                         base=self.config.get('base', 'ftp-base'),
                         username=self.config.get('base', 'ftp-username'),
                         password=self.config.get('base', 'ftp-password'),
                         pattern=self.config.get('base', 'ftp-pattern'),
                         max_retries=self.max_retries,
                         timeout=self.timeout)

    def run(self):
        self.input().move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(), format=TSV)


class BaseSingleFile(BaseTask):
    """
    Create a single compressed file of tarball.
    """
    date = luigi.DateParameter(default=datetime.date.today())

    def requires(self):
        return BasePaths(date=self.date)

    def run(self):
        with self.input().open() as handle:
            paths = [p.strip().decode('utf-8') for p in handle.readlines()]

        for path in paths:
            if not path.endswith("finc/latest"):
                continue

            realpath = os.path.realpath(path)
            self.logger.debug("found: %s", realpath)

            if realpath.endswith("tar.gz"):
                output = shellout(""" tar -xOzf "{input}" | pigz -c > {output}""", input=realpath)
                luigi.LocalTarget(output).move(self.output().path)
                break
            elif realpath.endswith("gz"):
                output = shellout("""cp "{input}" "{output}" """, input=realpath)
                luigi.LocalTarget(output).move(self.output().path)
                break
            else:
                raise RuntimeError('neither tarball nor gz: %s', realpath)
        else:
            raise RuntimeError('no finc/latest in %s', paths)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='ndj.gz'), format=Gzip)




#!/usr/bin/env python
# coding: utf-8
#
# Copyright 2019 by Leipzig University Library, http://ub.uni-leipzig.de
#                   The Finc Authors, http://finc.info
#                   Martin Czygan, <martin.czygan@uni-leipzig.de>
#                   Robert Schenk, <robert.schenk@uni-leipzig.de>
#
# This file is part of some open source application.
#
# Some open source application is free software: you can redistribute
# it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, either
# version 3 of the License, or (at your option) any later version.
#
# Some open source application is distributed in the hope that it will
# be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
#
# @license GPL-3.0+ <http://spdx.org/licenses/GPL-3.0+>

"""

Source: Filmakademie Baden-Württemberg (VK Film)
SID: 151
Ticket: #8700, #15871, #16012

"""

import io
import re
import sys

import marcx

from siskin.mappings import formats, roles
from siskin.utils import check_isbn, marc_build_field_008
from six.moves import html_parser


parser = html_parser.HTMLParser()


def get_field(field, tag):
    regexp = re.search('<.*? tag="%s">(.*)$' % tag, field)
    if regexp:
        _field = regexp.group(1)
        _field = parser.unescape(_field)
        return _field
    else:
        return ""


def get_subfield(field, tag, subfield):
    regexp = re.search('^<datafield.*tag="%s".*><subfield code="%s">(.*?)<\/subfield>' % (tag, subfield), field)
    if regexp:
        _field = regexp.group(1)
        _field = parser.unescape(_field)
        return _field
    else:
        return ""


# default input and output
inputfilename = "151_input.xml"
outputfilename = "151_output.mrc"

if len(sys.argv) == 3:
    inputfilename, outputfilename = sys.argv[1:]

inputfile = io.open(inputfilename, "r")
outputfile = io.open(outputfilename, "wb")

records = inputfile.read()
records = records.split("</record>")

for record in records:

    format = ""
    form = ""
    f001 = ""
    f020a = ""
    f100a = ""
    f100e = ""
    f245a = ""
    f260a = ""
    f260b = ""
    f260c = ""
    f300a = ""
    f300b = ""
    f300c = ""
    f650a = ""
    f700a = ""
    f700e = ""

    subjects = []
    persons = []
    f100 = []
    f700 = []

    marcrecord = marcx.Record(force_utf8=True)
    marcrecord.strict = False
    fields = re.split("(</controlfield>|</datafield>)", record)

    for field in fields:

        field = field.replace("\n", "")
        field = field.replace("</datafield>", "")
        field = field.replace("</controlfield>", "")

        form = get_field(field, "433")

        # format recognition
        if form and not format:
            
            regexp1 = re.search("\d\]?\sS\.", form)
            regexp2 = re.search("\d\]?\sSeit", form)
            regexp3 = re.search("\d\]?\sBl", form)
            regexp4 = re.search("\s?Illl?\.", form)
            regexp5 = re.search("[XVI],\s", form)
            regexp6 = re.search("^\d+\s[SsPp]", form)
            regexp7 = re.search("^DVD", form)
            regexp8 = re.search("^Blu.?-[Rr]ay", form)
            regexp9 = re.search("^H[DC] [Cc][Aa][Mm]", form)
            regexp10 = re.search("^HDCAM", form)
            regexp11 = re.search("[Bb]et.?-?[Cc]am", form)
            regexp12 = re.search("CD", form)
            regexp13 = re.search("[kKCc]asss?ette", form)
            regexp14 = re.search("^VHS", form)
            regexp15 = re.search("^Noten", form)
            regexp16 = re.search("^Losebl", form)
            regexp17 = re.search("^Film\s?\[", form)
            regexp18 = re.search("\d\smin", form)
            regexp19 = re.search("S\.\s\d+\s?-\s?\d+", form)

            if regexp1 or regexp2 or regexp3 or regexp4 or regexp5 or regexp6:
                format = "Book"
            elif regexp7:
                format = "DVD-Video"
            elif regexp8:
                format = "Blu-Ray-Disc"
            elif regexp9 or regexp10 or regexp11:
                format = "CD-Video"
            elif regexp12:
                format = "CD-Audio"
            elif regexp13 or regexp14:
                format = "Video-Cassette"
            elif regexp15:
                format = "Score"
            elif regexp16:
                format = "Loose-leaf"
            elif regexp17 or regexp18:
                format = "CD-Video"
            elif regexp19:
                format = "Article"                

        if f001 == "":
            f001 = get_field(field, "001")            

        if f020a == "":
            f020a = get_subfield(field, "540", "a")
        
        if f100a == "":
            f100a = get_subfield(field, "100", "a")
            if f100a != "":
                f100.append("a")
                f100.append(f100a)
                role = get_subfield(field, "100", "b")
                if role != "":
                    match = re.search("\[(.*?)\]", role)
                    if match:
                        role = match.group(1)
                        role = role.lower()
                        role = role.replace(".", "")
                        f1004 = roles.get(role, "")
                        if not f1004:
                            print("Missing role: %s." % role)
                        f100.append("4")
                        f100.append(f1004)
       
        if f245a == "":
            f245a = get_field(field, "331")
         
        if f260a == "":
            f260a = get_field(field, "410")

        if f260b == "":
            f260b = get_field(field, "412")

        if f260c == "":
            f260c = get_field(field, "425")

        if f300a == "":
            f300 = get_field(field, "433")
            # 335 S. : zahlr. Ill. ; 32 cm
            regexp1 = re.search("(.*)\s?:\s(.*);\s(.*)", f300)
            # 289 S.: Zahlr. Ill.
            regexp2 = re.search("(.*)\s?:\s(.*)", f300)
            # 106 S. ; 21 cm
            regexp3 = re.search("(.*)\s?;\s(.*)", f300)

            if regexp1:
                f300a, f300b, f300c = regexp1.groups()
            elif regexp2:
                f300a, f300b = regexp2.groups()
            elif regexp3:
                f300a, f300c = regexp3.groups()
            else:
                f300a = f300

        f650a = get_subfield(field, "710", "a")
        if f650a != "":
            subjects.append(f650a)

        regexp = re.search('tag="1\d\d"', field) # checks if there is a person field
        if regexp:
            for i in range(101, 197):  
                f700a = get_subfield(field, i, "a")
                if f700a != "":
                    f700.append("a")
                    f700.append(f700a)
                    role = get_subfield(field, i, "b")
                    if role != "":
                        match = re.search("\[(.*?)\]", role)
                        if match:
                            role = match.group(1)
                            role = role.lower()
                            role = role.replace(".", "")
                            f7004 = roles.get(role, "")
                            if not f7004:
                                print("Missing role: %s." % role)
                            f700.append("4")
                            f700.append(f7004)
                    persons.append(f700)
                    f700 = []
                    break

    if not f001:
        continue

    if not format:
        format = "Book"
        # print("Missing format: %s (Default = Book)" % form)

    # leader
    leader = formats[format]["Leader"]     
    marcrecord.leader = leader

    # identifier
    marcrecord.add("001", data="finc-151-" + f001)
    
    # access
    f007 = formats[format]["p007"]
    marcrecord.add("007", data=f007)

    # year, periodicity, language
    year = f260c
    periodicity = formats[format]["008"]
    language = ""
    f008 = marc_build_field_008(year, periodicity, language)
    marcrecord.add("008", data=f008)
    
    # ISBN
    f020a = check_isbn(f020a)
    marcrecord.add("020", a=f020a)

    # 1. creator
    marcrecord.add("100", subfields=f100)

    # title
    marcrecord.add("245", a=f245a)
    
    # imprint
    publisher = ["a", f260a, "b", f260b, "c", f260c]
    marcrecord.add("260", subfields=publisher)

    # physical description
    physicaldescription = ["a", f300a, "b", f300b, "c", f300c]
    marcrecord.add("300", subfields=physicaldescription)

    # RDA content
    f336b = formats[format]["336b"]
    marcrecord.add("336", b=f336b)

    # RDA carrier
    f338b = formats[format]["338b"]
    marcrecord.add("338", b=f338b)

    # subjects
    for f650a in subjects:
        marcrecord.add("650", a=f650a)

    # addional creator
    for f700 in persons:
        marcrecord.add("700", subfields=f700)

    # collection
    marcrecord.add("912", a="vkfilm")

    # SWB content
    f935c = formats[format]["935c"]
    marcrecord.add("935", c=f935c)

    # Ansigelung 
    collections = ["a", f001, "b", "151", "c", "sid-151-col-filmakademiebawue"]
    marcrecord.add("980", subfields=collections)

    outputfile.write(marcrecord.as_marc())

inputfile.close()
outputfile.close()



#!/usr/bin/env python
# coding: utf-8
#
# Copyright 2019 by Leipzig University Library, http://ub.uni-leipzig.de
#                   The Finc Authors, http://finc.info
#                   Martin Czygan, <martin.czygan@uni-leipzig.de>
#                   Robert Schenk, <robert.schenk@uni-leipzig.de>
#
# This file is part of some open source application.
#
# Some open source application is free software: you can redistribute
# it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, either
# version 3 of the License, or (at your option) any later version.
#
# Some open source application is distributed in the hope that it will
# be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
#
# @license GPL-3.0+ <http://spdx.org/licenses/GPL-3.0+>

"""

Source: Wolfenbütteler Bibliographie zur Geschichte des Buchwesens
SID: 173
Ticket: #13640

"""

import re
import sys

import marcx
import pymarc
from siskin.utils import marc_clean_record


def get_field(record, field, subfield):
    try:
        return record[field][subfield]
    except:
        return ""


def get_inds_for_field_245(title):
    """
    Checks if the title starts with an article and returns the appropriate.
    """
    
    if title.startswith("A "):
        return " 2"
    elif title.startswith("Un "):
        return " 3"
    elif title.startswith("Ein "):
        return " 4"
    elif title.startswith("Der "):
        return " 4"
    elif title.startswith("Die "):
        return " 4"
    elif title.startswith("Das "):
        return " 4"
    elif title.startswith("The "):
        return " 4"
    elif title.startswith("Une "):
        return " 4"
    elif title.startswith("Una "):
        return " 4"
    elif title.startswith("Eine "):
        return " 5"
    else:
        return "  "


inputfilename = "173_input.mrc"
outputfilename = "173_output.mrc"

if len(sys.argv) == 3:
    inputfilename, outputfilename = sys.argv[1:]

inputfile = open(inputfilename, "rb")
outputfile = open(outputfilename, "wb")

reader = pymarc.MARCReader(inputfile, force_utf8=True)

for record in reader:

    record = marcx.Record.from_record(record)
    record.force_utf8 = True
    record.strict = False

    try:
        parent_type = record["773"]["7"]
    except:
        parent_type = ""

    # Leader
    if parent_type == "nnam":
        f935b = "druck"
        f935c = ""
        f008 = ""
        leader = "     " + record.leader[5:]
        record.leader = leader
    elif parent_type == "nnas":
        f935b = "druck"
        f935c = "text"
        f008 = "                     p"
        leader = record.leader[8:]
        leader = "     nab" + leader
        record.leader = leader
    else:
        f935b = "druck"
        f935c = ""
        f008 = ""
        leader = "     " + record.leader[5:]
        record.leader = leader

    # Identifikator
    f001 = record["001"].data
    regexp = re.search("edoc/(.*)", f001)
    if regexp:
        f001 = regexp.group(1)
    else:
        sys.exit("Die ID konnte nicht verarbeitet werden: " + f001)
    f001 = f001.replace(":", "")
    record.remove_fields("001")
    record.add("001", data="finc-173-%s" % f001)

    # Zugangsformat
    record.add("007", data="tu")

    # Periodizität
    record.add("008", data=f008)

    # Titel
    f245a = get_field(record, "245", "a")
    if f245a:
        record.remove_fields("245")
        f245a = re.sub("\s?~?\\\dagger\s?", "", f245a)
        inds = get_inds_for_field_245(f245a)
        record.add("245", a=f245a, indicators=inds)
    else:
        continue

    # Anmerkung
    f553a = get_field(record, "553", "a")
    if f553a:
        f553a = re.sub("\s?~?\\\dagger", "", f553a)
        record.remove_fields("553")
        record.add("553", a=f553a)

    # SWB-Format
    record.add("935", b=f935b, c=f935c)

    # Ansigelung und Kollektion
    collections = ["a", f001, "b", "173", "c", "sid-173-col-buchwesen"]
    record.add("980", subfields=collections)

    marc_clean_record(record)
    outputfile.write(record.as_marc())

inputfile.close()
outputfile.close()



#!/usr/bin/env python
# coding: utf-8
#
# Copyright 2019 by Leipzig University Library, http://ub.uni-leipzig.de
#                   The Finc Authors, http://finc.info
#                   Martin Czygan, <martin.czygan@uni-leipzig.de>
#                   Robert Schenk, <robert.schenk@uni-leipzig.de>
#
# This file is part of some open source application.
#
# Some open source application is free software: you can redistribute
# it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, either
# version 3 of the License, or (at your option) any later version.
#
# Some open source application is distributed in the hope that it will
# be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
#
# @license GPL-3.0+ <http://spdx.org/licenses/GPL-3.0+>
"""

Source: Kieler Beiträge zur Filmmusikforschung
SID: 101
Ticket: #7967, #9371, #10831, #12048, #12967, #14649, #15882

"""

import io
import re
import sys

import marcx
import base64
import xlrd

from siskin.mappings import formats
from siskin.utils import check_issn, marc_build_field_008


inputfilename = "101_input.xlsx"
outputfilename = "101_output.mrc"

if len(sys.argv) == 3:
    inputfilename, outputfilename = sys.argv[1:]

outputfile = open(outputfilename, "wb")

workbook = xlrd.open_workbook(inputfilename)
sheet = workbook.sheet_by_name("Tabelle1")

format = "Article"
access = "electronic"

"""
authors = 0
rft_atitle = 1
rft_jtitle = 2 
rft_pub = 3
rft_genre = 4 
rft_issn = 5
rft_issue = 6
rft_volume = 7
rft_date = 8
rft_spage = 9
rft_epage = 10
rft_tpages = 11
rft_pages = 12
url = 13
"""

for row in range(1, sheet.nrows):

    csvrecord = sheet.row_values(row)

    marcrecord = marcx.Record(force_utf8=True)
    marcrecord.strict = False

    # Leader
    leader = formats[format]["Leader"]
    marcrecord.leader = leader

    # Identifikator
    try:
        url = csvrecord[13]
    except:
       continue
    url = bytes(url, "utf-8")
    url = base64.b64encode(url)
    f001 = url.decode("utf-8").rstrip("=")
    marcrecord.add("001", data="finc-101-" + f001)

    # Zugang
    if access == "electronic":
        f007 = formats[format]["e007"]
    else:
        f007 = formats[format]["p007"]
    marcrecord.add("007", data=f007)

    # Feld 008
    year = csvrecord[8]
    year = str(year).rstrip("0").rstrip(".") # rstrip(".0") doesn't work equally here
    periodicity = formats[format]["008"]
    language = "ger"
    f008 = marc_build_field_008(year, periodicity, language)
    marcrecord.add("008", data=f008)

    # ISSN
    issn = csvrecord[5]
    issn = str(issn)
    f022a = check_issn(issn)
    marcrecord.add("022", a=f022a)

    # Sprache
    marcrecord.add("041", a="ger")

    # 1. Urheber
    authors = csvrecord[0]
    if authors:
        authors = authors.split("; ")
        f100a = authors[0]
        marcrecord.add("100", a=f100a)

    # Titel
    title = csvrecord[1]
    if ": " in title:
        titles = title.split(": ")
        f245a = titles[0]
        f245b = ""
        for title in titles[1:]:
            f245b = f245b + title + " : "
    else:
        f245a = title
        f245b = ""
    f245a = f245a.strip()
    f245b = f245b.rstrip(" :")
    f245b = f245b.strip()
    marcrecord.add("245", a=f245a, b=f245b)

    # Erscheinungsvermerk
    year = csvrecord[8]
    f260c = str(year).rstrip("0").rstrip(".") # rstrip(".0") doesn't work equally here
    subfields = ("a", "Kiel", "b", "Kieler Gesellschaft für Filmmusikforschung", "c", f260c)
    marcrecord.add("260", subfields=subfields)

    # Umfang
    pages = str(csvrecord[12])
    extent = pages.split("-")
    if len(extent) == 2:
        start = int(extent[0])
        end = int(extent[1])
        f300a = str(end - start + 1) + " Seiten"
    else:
        f300a = ""
    marcrecord.add("300", a=f300a)

    # weitere Urheber
    if authors:
        if len(authors) > 1:
            for f700a in authors[1:]:
                marcrecord.add("700", a=f700a)

    # Verweis auf Elternelement
    f773t = csvrecord[2]
    year = str(csvrecord[8]).rstrip(".0")
    pages = str(csvrecord[12])
    issue = str(csvrecord[6]).rstrip(".0")
    f773g = "(" + year + "), Heft: " + issue + ", S. " + pages 
    marcrecord.add("773", t=f773t, g=f773g)

    # Link
    f856u = csvrecord[13]
    marcrecord.add("856", q="text/pdf", _3="Link zur Ressource", u=f856u)

    # Ansigelung
    collections = ["a", f001, "b", "101", "c", "sid-101-col-kielfilm"]
    marcrecord.add("980", subfields=collections)
    outputfile.write(marcrecord.as_marc())

outputfile.close()
